/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include <qpe/qpeapplication.h>
#include <qpe/config.h>
#include <qpe/network.h>
#ifdef QWS
#include <qpe/qcopenvelope_qws.h>
#endif
#include <qpe/global.h>
#include <qpe/process.h>
#include "../proxiespage.h"

#include <qbuttongroup.h>
#include <qvbox.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qtabwidget.h>
#include <qbuffer.h>
#include <qspinbox.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include "usblan.h"
#include "usblanstate.h"
#include <qlayout.h>

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/vfs.h>
#include <mntent.h>
#include <unistd.h>
#include <sys/types.h>

#define USE_SCHEMES

#ifdef USE_SCHEMES
static QString toSchemeId(const QString& s)
{
    QString r;
    for (int i=0; i<(int)s.length(); i++) {
	char c = s[i].lower().latin1();
	if ( c>='a' && c<='z' || i>0 && c>='0' && c<='9' )
	    r += s[i];
	//else
	    //r += "_U" + QString::number(s[i].unicode()) + "_";
    }
    return r;
}
#endif

class SchemeChanger : public QObject {
    Q_OBJECT
public:
    SchemeChanger(QObject* parent) :
	QObject(parent)
    {
    }

    static void changeUSBLanScheme(const QString& usblan_or_scheme)
    {
#ifdef USE_SCHEMES	
	Config c("Network");
	c.setGroup("USBLan");
	QString s = toSchemeId(usblan_or_scheme);
	c.writeEntry("Scheme",s);
	system("cardctl scheme " + s);
#else	
	QStringList srv = Network::choices();
	for (QStringList::ConstIterator it=srv.begin(); it!=srv.end(); ++it) {
	    QString t = Network::serviceType(*it);
	    if ( t == "usblan" ) {
		Config cfg(*it,Config::File);
		cfg.setGroup("Info");
		QString n = cfg.readEntry("Name");
		if ( n == usblan_or_scheme ) {
		    USBLan::writeNetworkOpts( cfg );
		}
	    }
	}	
#endif	
#ifndef QT_NO_COP
	QCopEnvelope e("QPE/Network","choicesChanged()");
#endif
    }

public slots:
    void changeUSBLanSchemeSlot(const QString& usblan_or_scheme)
    {
	changeUSBLanScheme(usblan_or_scheme);
    }
};

class WepKeyValidator : public QValidator {
public:
    WepKeyValidator(QWidget* parent) :
	QValidator(parent)
    {
    }

    State validate( QString& key, int& curs ) const
    {
	QString k;
	int hexes=0;
	int ncurs=0;
	for (int i=0; i<(int)key.length(); i++) {
	    char c=key[i].lower().latin1();
	    if ( c>='0' && c<='9' || c>='a' && c<='z' ) {
		if ( hexes == 2 ) {
		    hexes = 0;
		    k += ':';
		    if ( i<curs ) ncurs++;
		}
		k += c;
		hexes++;
		if ( i<curs ) ncurs++;
	    } else if ( c == ':' && hexes==2 ) {
		hexes = 0;
		k += c;
		if ( i<curs ) ncurs++;
	    }
	}
	key = k;
	curs = ncurs;
	return Acceptable;
    }

    //void fixup( QString& key ) const
    //{
    //}
};

USBLan::USBLan(Config& cfg, QWidget* parent) :
    USBLanBase(parent,0,TRUE),
    config(cfg)
{
    ipGroup = new QButtonGroup( this );
    ipGroup->hide();
    ipGroup->insert( manualIP );
    ipGroup->insert( autoIP );
    connect( ipGroup, SIGNAL(clicked(int)), this, SLOT(ipSelect(int)) );
    ipGroup->setButton( 1 );

    QBoxLayout *proxyLayout  = new QVBoxLayout( tab_2 ); 
    proxies = new ProxiesPage( tab_2 );
    proxyLayout->addWidget( proxies );

    connect(wep_passphrase, SIGNAL(textChanged(const QString&)), this, SLOT(chooseDefaultWepKey()));
    connect(wep_key0, SIGNAL(textChanged(const QString&)), this, SLOT(chooseDefaultWepKey()));
    connect(wep_key1, SIGNAL(textChanged(const QString&)), this, SLOT(chooseDefaultWepKey()));
    connect(wep_key2, SIGNAL(textChanged(const QString&)), this, SLOT(chooseDefaultWepKey()));
    connect(wep_key3, SIGNAL(textChanged(const QString&)), this, SLOT(chooseDefaultWepKey()));
    connect(wep_type, SIGNAL(activated(int)), this, SLOT(wepTypeChanged(int)));
    QValidator* key = new WepKeyValidator(this);
    wep_key0->setValidator(key);
    wep_key1->setValidator(key);
    wep_key2->setValidator(key);
    wep_key3->setValidator(key);

    readConfig();
}

void USBLan::wepTypeChanged(int i)
{
    wep_choice->setEnabled(i>0);
}

void USBLan::chooseDefaultWepKey()
{
    if ( sender() == wep_passphrase )
	wep_passphrase_on->setChecked(TRUE);
    else if ( sender() == wep_key0 )
	wep_key0_on->setChecked(TRUE);
    else if ( sender() == wep_key1 )
	wep_key1_on->setChecked(TRUE);
    else if ( sender() == wep_key2 )
	wep_key2_on->setChecked(TRUE);
    else if ( sender() == wep_key3 )
	wep_key3_on->setChecked(TRUE);
}

void USBLan::ipSelect( int id )
{
    tabs->setTabEnabled( tcpip, id==0 );
}

void USBLan::readConfig()
{


    config.setGroup("Info");
    acname->setText(config.readEntry("Name","USBLAN"));

    config.setGroup("Properties");

    int id = config.readEntry("DHCP","y") != "n";
    ipGroup->setButton( id );
    ipSelect( id );

    WLanEnable->setChecked(config.readEntry("WLAN","y") != "n");
    CardNames->setText( config.readEntry("CardNames","prism2_usb Pegasus"));

    ip->setText( config.readEntry("IPADDR") );
    subnet->setText( config.readEntry("NETMASK","255.255.255.0") );
    broadcast->setText( config.readEntry("BROADCAST","") );
    gateway->setText( config.readEntry("GATEWAY") );
    dns1->setText( config.readEntry("DNS_1") );
    dns2->setText( config.readEntry("DNS_2") );

    QString ssid = config.readEntry("SSID");
    if ( !ssid.isEmpty() ) {
	usbwlan_ssid->insertItem(ssid);
	usbwlan_ssid->setCurrentItem(usbwlan_ssid->count()-1);
    }
    bool ah=config.readEntry("IS_ADHOC").lower()=="y";
    (ah ? usbwlan_adhoc : usbwlan_infrastructure)->setChecked(TRUE);
    usbwlan_channel->setValue(config.readNumEntry("CHANNEL"));
    wep_passphrase->setText(config.readEntry("PRIV_GENSTR"));
    wep_key0->setText(config.readEntry("dot11WEPDefaultKey0"));
    wep_key1->setText(config.readEntry("dot11WEPDefaultKey1"));
    wep_key2->setText(config.readEntry("dot11WEPDefaultKey2"));
    wep_key3->setText(config.readEntry("dot11WEPDefaultKey3"));
    int weptype = config.readEntry("PRIV_KEY128")=="true" ? 2 : 1;
    QString wep = config.readEntry("WEP");
    if ( wep == "PP" )
	wep_passphrase_on->setChecked(TRUE);
    else if ( wep == "K0" )
	wep_key0_on->setChecked(TRUE);
    else if ( wep == "K1" )
	wep_key1_on->setChecked(TRUE);
    else if ( wep == "K2" )
	wep_key2_on->setChecked(TRUE);
    else if ( wep == "K3" )
	wep_key3_on->setChecked(TRUE);
    else
	weptype=0;
    wep_type->setCurrentItem(weptype);
    wepTypeChanged(weptype);

    config.setGroup("Proxy");
    proxies->readConfig( config );
}

bool USBLan::writeConfig()
{
    QString nm = acname->text();

    // make sure we don't duplicate the name
    QStringList srv = Network::choices();
    for (QStringList::ConstIterator it=srv.begin(); it!=srv.end(); ++it) {
	QString t = Network::serviceType(*it);
	if ( t == "usblan" ) {
	    Config cfg(*it,Config::File);
	    cfg.setGroup("Info");
	    QString n = cfg.readEntry("Name");
/*
	    if ( (n == nm

#ifdef USE_SCHEMES
		|| toSchemeId(n) == toSchemeId(nm)
#endif

		  ) && (nm != config.readEntry("NAME")) //cfg != config
	    ) {
		QMessageBox::warning(0, tr("USB LAN Setup"), 
				     tr( "This name already\nexists. Please choose a\ndifferent name.") );
		return FALSE;
	    }
*/
	}
    }

#ifdef USE_SCHEMES    
    QString scheme;
#endif
    {
	config.setGroup("Info");
	config.writeEntry("Name",nm);
#ifdef USE_SCHEMES    
	scheme = toSchemeId(nm);
	if ( scheme.isEmpty() )
	    scheme = "*"; // The default config
#endif    
	config.setGroup("Properties");

	config.writeEntry("WLAN",WLanEnable->isChecked() ? "y" : "n");
	config.writeEntry("CardNames",CardNames->text());

	config.writeEntry("DHCP",autoIP->isChecked() ? "y" : "n");
	config.writeEntry("IPADDR",ip->text());
	config.writeEntry("NETMASK",subnet->text());
	config.writeEntry("BROADCAST",broadcast->text());
	config.writeEntry("GATEWAY",gateway->text());
	config.writeEntry("DNS_1",dns1->text());
	config.writeEntry("DNS_2",dns2->text());

	config.writeEntry("USBWLAN_ENABLE","y");
	config.writeEntry("SSID",usbwlan_ssid->currentText());
	config.writeEntry("IS_ADHOC",usbwlan_adhoc->isChecked() ? "y" : "n");
	config.writeEntry("PRIV_KEY128",wep_type->currentItem()==2 ? "true" : "false");
	config.writeEntry("CHANNEL",usbwlan_channel->value());
	config.writeEntry("PRIV_GENSTR", wep_passphrase->text());
	config.writeEntry("dot11PrivacyInvoked", wep_type->currentItem()==0 ? "false" : "true");
	config.writeEntry("AuthType", wep_type->currentItem()==0 ? "opensystem" : "sharedkey");
	int defkey = wep_choice->id(wep_choice->selected());
	if ( defkey < 1 || defkey > 4 )
	    defkey = 1;
	config.writeEntry("dot11WEPDefaultKeyID",defkey-1);
	config.writeEntry("dot11WEPDefaultKey0", wep_key0->text());
	config.writeEntry("dot11WEPDefaultKey1", wep_key1->text());
	config.writeEntry("dot11WEPDefaultKey2", wep_key2->text());
	config.writeEntry("dot11WEPDefaultKey3", wep_key3->text());
	QString wep;
	if ( wep_passphrase_on->isChecked() )
	    wep = "PP";
	else if ( wep_key0_on->isChecked() )
	    wep = "K0";
	else if ( wep_key1_on->isChecked() )
	    wep = "K1";
	else if ( wep_key2_on->isChecked() )
	    wep = "K2";
	else if ( wep_key3_on->isChecked() )
	    wep = "K3";
	else
	    wep = "NO";
	config.writeEntry("WEP",wep);

	config.setGroup("Proxy");
	proxies->writeConfig( config );
    }

#ifdef USE_SCHEMES    
    return writeNetworkOpts( config, scheme );
#else
    return TRUE;
#endif    
}

bool USBLan::writeNetworkOpts( Config &config, QString scheme )
{
    // PCMCIA card will have "CardType = network" entry
    config.setGroup("Properties");
    QString cardType = config.readEntry("CardType");

    if (cardType == "usblan") // No tr
        return writeUSBLanNetworkOpts(config, scheme);
    else
	return writeBuiltinNetworkOpts(config, scheme);
}

bool USBLan::writeUSBLanNetworkOpts( Config& /*config*/, QString /*scheme*/ )
{
	return TRUE;
}

bool USBLan::writeBuiltinNetworkOpts( Config& /*config*/, QString /*scheme*/ )
{
    // Since no standard exists for configuring networks, this
    // function must be implemented by the system integrator.
    //
    // The procedure is:
    //  1. write the network configuration file to a tmp file.
    //  2. stop the network.
    //  3. move the tmp file to the network config file.
    //  4. start the network.

    return TRUE;
}

void USBLan::accept()
{
    if ( writeConfig() )
	QDialog::accept();
}

static QStringList findUSBLans(const QString& skip=QString::null)
{
    QStringList srv = Network::choices();
    QStringList usblans;
    for (QStringList::ConstIterator it=srv.begin(); it!=srv.end(); ++it) {
	QString t = Network::serviceType(*it);
	if ( t == "usblan" ) {
	    Config cfg(*it,Config::File);
	    cfg.setGroup("Info");
	    QString n = cfg.readEntry("Name");
	    if ( n != skip )
		usblans.append(n);
	}
    }
    return usblans;
}

static QString findScheme(const QStringList& usblans)
{
    QString scheme;
    if ( usblans.count() > 0 ) {
	{
	    Config c("Network");
	    c.setGroup("USBLan");
	    scheme = c.readEntry("Scheme");
	}
	bool found=FALSE;
	for (QStringList::ConstIterator it=usblans.begin(); !found && it!=usblans.end(); ++it) {
	    if ( toSchemeId(*it) == scheme )
		found = TRUE;
	}
	if ( !found )
	    SchemeChanger::changeUSBLanScheme( usblans[0] );
    } else {
	{
	    Config c("Network");
	    c.setGroup("USBLan");
	    c.writeEntry("Scheme","default"); // No tr
	}
	SchemeChanger::changeUSBLanScheme( "default" ); // No tr
    }
    return scheme;
}


USBLanImpl::USBLanImpl()
{
}

USBLanImpl::~USBLanImpl()
{
}

QRESULT USBLanImpl::queryInterface( const QUuid &uuid, QUnknownInterface **iface )
{
    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = this;
    else if ( uuid == IID_Network )
	*iface = this;
    else
	return QS_FALSE;

    (*iface)->addRef();
    return QS_OK;
}

Q_EXPORT_INTERFACE()
{
    Q_CREATE_INSTANCE( USBLanImpl )
}

bool USBLanImpl::doProperties( QWidget *parent, Config& cfg )
{
    USBLan dialog(cfg,parent);
    dialog.showMaximized();
    return dialog.exec();
}

bool USBLanImpl::create( Config& cfg )
{
    QStringList usblans = findUSBLans();
    QString nm = cfg.readEntry("Name");
    int n=1;
    while ( usblans.contains(nm) ) {
	nm = "USBLAN " + QString::number(++n);
    }
    cfg.writeEntry("Name",nm);
    return TRUE;
}

bool USBLanImpl::remove( Config& cfg )
{
#ifdef USE_SCHEMES
    // Switch to another
    cfg.setGroup("Info");
    QStringList usblans = findUSBLans(cfg.readEntry("Name"));
    findScheme(usblans);
#endif
    return TRUE;
}

int USBLanImpl::exec(const QString& arg, QTextStream& log)
{
	log << arg << endl;
	int rc = system((!isRoot ? "sudo " : "") + arg);
	if (rc == 0)
		log << "success!" << endl;
	else
		log << "failed!" << endl;

	return rc;
}

void USBLanImpl::progress(const QString& p, int flag)
{
#ifndef QT_NO_COP
	QCopEnvelope e("QPE/Network", "progress(QString, int)");
	e << p << flag;
#endif
}

bool USBLanImpl::start( Config& cfg )
{

	isRoot = (geteuid() == 0);

	// start USBLan
	QFile tmp("/tmp/usblan.log");
	if (!tmp.open(IO_WriteOnly | IO_Append))
		return false;

	QTextStream log( &tmp );
	log << "Starting USBLAN..." << endl;


	QString cn = cfg.readEntry("CardNames");
	if ( cn.isEmpty() )
	  return FALSE;
	QStringList cns = QStringList::split(" ", cn, FALSE);
	bool isWlan = (cfg.readEntry("WLAN") != "n");


	if(isWlan){	// get wlan device name
	  for(int i = 0; i < 3; i++){
	    QString result;
	    QString wlandev;
	    wlandev =  "wlan" + QString().setNum(i);
	    Process p(QStringList() << "ethtool" << "-i" << wlandev);
	    p.exec("", result);
	    
	    log << result;
	    
	    for (QStringList::Iterator it = cns.begin(); it != cns.end(); ++it ){
	      if( result.find(*it, 0, FALSE) != -1)
		DeviceName = "wlan"+QString().setNum(i);
	    }
	  }
	}else{           // get eth device name (0-3)

	  for(int i = 0; i < 3; i++){
	    QString result;
	    QString ethdev;
	    ethdev =  "eth" + QString().setNum(i);
	    Process p(QStringList() << "ethtool" << "-i" << ethdev);
	    p.exec("", result);
	    
	    log << result;
	    
	    for (QStringList::Iterator it = cns.begin(); it != cns.end(); ++it ){
	      if( result.find(*it, 0, FALSE) != -1)
		DeviceName = "eth"+QString().setNum(i);
	    }
	  }
	}

	log << DeviceName;

	QString command;

	cfg.setGroup("Properties");

	if(cfg.readEntry("WLAN") == "y"){
	  QString result;
	  {
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "lnxreq_ifstate" << "ifstate=enable");
	    p.exec("", result);
	    log << result;
	  }
	  if(!result.contains("resultcode=success")){
	     return FALSE;
	  }

	  {
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "lnxreq_hostwep" 
		      << "decrypt=true" << "encrypt=true");
	    p.exec("", result);
	    log << result;
	  }

	  QString wepkey;
	  {
	    wepkey = "mibattribute=dot11WEPDefaultKeyID=%1";
	    wepkey = wepkey.arg(cfg.readEntry("dot11WEPDefaultKeyID"));
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "dot11req_mibset" << wepkey);
	    p.exec("", result);
	    log << wepkey << result;
	  } 
	  if(!result.contains("resultcode=success")){
	     return FALSE;
	  }

	  {
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "dot11req_mibset" 
		      << "mibattribute=dot11ExcludeUnencrypted=true");
	    p.exec("", result);
	    log << result;
	  } 
	  if(!result.contains("resultcode=success")){
	     return FALSE;
	  }

	  {
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "dot11req_mibset" 
		      << "mibattribute=dot11PrivacyInvoked=true");
	    p.exec("", result);
	    log << result;
	  } 
	  if(!result.contains("resultcode=success")){
	     return FALSE;
	  }

	  {
	    wepkey = "mibattribute=dot11WEPDefaultKey0=%1";
	    wepkey = wepkey.arg(cfg.readEntry("dot11WEPDefaultKey0"));
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "dot11req_mibset" << wepkey);
	    p.exec("", result);
	    log << wepkey << result;
	  } 
	  {
	    wepkey = "mibattribute=dot11WEPDefaultKey1=%1";
	    wepkey = wepkey.arg(cfg.readEntry("dot11WEPDefaultKey1"));
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "dot11req_mibset" << wepkey);
	    p.exec("", result);
	    log << result;
	  } 
	  {
	    wepkey = "mibattribute=dot11WEPDefaultKey2=%1";
	    wepkey = wepkey.arg(cfg.readEntry("dot11WEPDefaultKey2"));
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "dot11req_mibset" << wepkey);
	    p.exec("", result);
	    log << result;
	  } 
	  {
	    wepkey = "mibattribute=dot11WEPDefaultKey3=%1";
	    wepkey = wepkey.arg(cfg.readEntry("dot11WEPDefaultKey3"));
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "dot11req_mibset" << wepkey);
	    p.exec("", result);
	    log << result;
	  } 

	  if(cfg.readEntry("WEP") == "PP"){
	    QString passkey = cfg.readEntry("PRIV_GENSTR");
	    if (passkey == "")
	      return FALSE;

	    wepkey = "mibattribute=dot11WEPDefaultKey0=%1";

	    if(cfg.readEntry("PRIV_KEY128") == "true"){
	      Process p(QStringList() << "nwepgen" << passkey << "13");
	      p.exec("",result);
	      log << result;

	      wepkey = wepkey.arg(result.left(38));

	    }else {
	      Process p(QStringList() << "nwepgen" << passkey << "5");
	      p.exec("",result);
	      log << result;

	      wepkey = wepkey.arg(result.left(14));
	    }

	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "dot11req_mibset" << wepkey);
	    p.exec("", result);
	    log << wepkey << result;
	  }

	  {
	    QString authtype,ssid;
	    authtype = "authtype=%1";
	    authtype = authtype.arg(cfg.readEntry("AuthType"));
	    ssid = "ssid=%1";
	    ssid = ssid.arg(cfg.readEntry("SSID"));
	    Process p(QStringList() << "wlanctl-ng" << DeviceName << "lnxreq_autojoin" << ssid << authtype);
	    p.exec("", result);
	    log << result;
	  } 
	  if(!result.contains("resultcode=success")){
	     return FALSE;
	  }
	  
	  
/*
#wlanctl-ng wlan0 lnxreq_ifstate ifstate=enable
#wlanctl-ng wlan0 lnxreq_hostwep decrypt=true encrypt=true
#wlanctl-ng wlan0 dot11req_mibset mibattribute=dot11WEPDefaultKeyID=0
#wlanctl-ng wlan0 dot11req_mibset mibattribute=dot11ExcludeUnencrypted=true
#wlanctl-ng wlan0 dot11req_mibset mibattribute=dot11PrivacyInvoked=true
#wlanctl-ng wlan0 dot11req_mibset mibattribute=dot11WEPDefaultKey0=55:55:55:55:55:55:55:55:55:55:55:55:55
wlanctl-ng wlan0 lnxreq_autojoin ssid=piro_wireless authtype=opensystem
*/



	}

	// set up IP
	progress(QT_TR_NOOP("Configuring network"), 0);
	command = "ifconfig %1 %2 netmask %3";
	QString ip = cfg.readEntry("IPADDR");
	QString mask = cfg.readEntry("NETMASK","255.255.255.0");

	QString broadcast = cfg.readEntry("BROADCAST","");
	if (broadcast != "")
		command += " broadcast %4";

	if (exec(command.arg(DeviceName).arg(ip).arg(mask).arg(broadcast), log))
		return false;

	system("sleep 1");

	// add route
	QString gw = cfg.readEntry("GATEWAY");
	if (gw != "")
	{
		command = "route add default gw %1";
		exec(command.arg(gw), log);	// ignore result
	}

	// add DNS
	QString dns1 =  cfg.readEntry("DNS_1");
	QString dns2 = cfg.readEntry("DNS_2");

	exec("cp -f /etc/resolv.conf /etc/resolv.conf.bak", log);

	if (dns1 != "" || dns2 != "")
	{
		log << "Adding DNS entries..." << endl;

		QFile resolve("/etc/resolv.conf");
		if (!resolve.open(IO_WriteOnly | IO_Truncate))
			return false;

		QTextStream out(&resolve);
		
		if (dns1 != "")
		{
			out << "nameserver " << dns1 << endl;
			log << "nameserver " << dns1 << endl;
		}

		if (dns2 != "")
		{
			out << "nameserver " << dns2 << endl;
			log << "nameserver " << dns2 << endl;
		}

		if (dns1 == "" && dns2 == "")
			out << "nameserver " << "127.0.0.1" << endl;

		log << "success!" << endl;
		resolve.close();
	}

	progress(QT_TR_NOOP("Connected"), 5);
	tmp.close();

	return true;

}

bool USBLanImpl::stop( Config& cfg )
{
/*
    // Not implemented
    return !isActive(cfg);
*/

	QFile tmp("/tmp/usblan.log");
	if (!tmp.open(IO_WriteOnly | IO_Append))
		return false;

	QTextStream log( &tmp );

	QString command;

	command = "ifconfig %1 down";
	log << "Stopping USBLAN..." << endl;
	exec(command.arg(DeviceName), log);

	QString dns1 =  cfg.readEntry("DNS_1");
	QString dns2 = cfg.readEntry("DNS_2");



	// restore config if needed
	exec("mv -f /etc/resolv.conf.bak /etc/resolv.conf", log);

	DeviceName = "";
	return !isActive(cfg);
}

class USBLanStateImpl : public USBLanState {
    Q_OBJECT
public:
    USBLanStateImpl(QWidget* parent, const QString &dev) : USBLanState(parent), device(dev)
    {
	QStringList usblans = findUSBLans();
	QString scheme = findScheme(usblans);
	if ( usblans.count() > 1 ) {
	    services->insertStringList(usblans);
	    services->setCurrentItem(usblans.findIndex(scheme));
	    SchemeChanger* sc = new SchemeChanger(this);
	    QObject::connect(services,SIGNAL(activated(const QString&)),
		sc,SLOT(changeUSBLanSchemeSlot(const QString&)));
	} else {
	    services->hide();
	    services_label->hide();
	}
	if ( !device[device.length()-1].isDigit() )
	    device += "0";
	tid = startTimer(1000);
	updateDisplay();
    }

    void timerEvent(QTimerEvent*)
    {
	updateDisplay();
	if ( ipaddress->text()[0] != 'e' ) {
	    // slow down
	    killTimer(tid);
	    tid = startTimer(5000);
	}
    }

    bool ok() const
    {
	return !ipaddress->text().isEmpty();
    }

private slots:
    void updateDisplay()
    {
	QString ipaddr;
	Process p(QStringList() << "ifconfig" << device);
	if ( p.exec( "", ipaddr ) ) {
	    ipaddr.replace(QRegExp(".*inet addr:"),"");
	    ipaddr.replace(QRegExp(" .*"),"");
	    ipaddress->setText(ipaddr);
	} else {
	    ipaddress->setText("");
	}
    }

private:
    int tid;
    QString device;
};

QWidget* USBLanImpl::addStateWidget( QWidget* parent, Config& cfg ) const
{
    if ( isActive(cfg) ) {
	cfg.setGroup( "Properties" );
	QString dev = cfg.readEntry( "Device" );
	USBLanStateImpl* state = new USBLanStateImpl(parent, dev);
	if ( state->ok() )
	    return state;
	delete state;
    }
    return 0;
}

bool USBLanImpl::isActive( Config& /*cfg*/ ) const
{
  QString result;
  Process p(QStringList() << "ifconfig");
  p.exec("", result);
  
  if (DeviceName == "")
    return FALSE;

  return result.contains(DeviceName);
}

bool USBLanImpl::isAvailable( Config& cfg ) const
{
    QString ct = cardType(cfg);

    if ( ct.isEmpty() )
	return FALSE;

    QString result;
    QStringList cts = QStringList::split(" ", ct, FALSE);

    Process p(QStringList() << "lsmod");
    p.exec("", result);

    for (QStringList::Iterator it = cts.begin(); it != cts.end(); ++it ){
      if( result.find(*it, 0, FALSE) != -1)
	return TRUE;
    }
    return FALSE;
    

}


#include "usblan.moc"
